import { commonConvertMap } from '../utils/convert-helper'

export default {
	inject: {
		$$mapRoot: {
			default: null,
		},
	},
	data() {
		return {
			unwatchFns: [],
		}
	},
	async created() {
		// 地图slot的组件，需要通过 $$amapRoot 获取地图实例
		if (this.$$mapRoot) {
			this.$amap = this.$$mapRoot.$amap
			this.register()
		}
	},
	methods: {
		register() {
			const res = this._initComponent && this._initComponent()
			if (res && res.then) {
				res.then(instance => this.registerRest(instance))
			} else this.registerRest(res)
		},

		registerRest(instance) {
			if (!this.$amapComponent && instance) this.$amapComponent = instance
			this.registerEvents()
			this.setPropWatchers()
			if (this.events && this.events.init) this.events.init(this.$amapComponent)
		},

		//获取props处理函数
		getHandlerFun(prop) {
			if (this._handlers && this._handlers[prop]) {
				return this._handlers[prop]
			}
		},
		//传入的值转化为amap需要的属性值
		convertProps() {
			const props = {}
			if (this.$amap) props.map = this.$amap
			const {
				$options: { propsData = {} },
			} = this
			return Object.keys(propsData).reduce((res, _key) => {
				let key = _key
				let propsValue = this.convertSignalProp(key, propsData[key])
				if (propsValue === undefined) return res
				props[key] = propsValue
				return res
			}, props)
		},
		convertSignalProp(key, sourceData) {
			if (this._converters && this._converters[key]) {
				return this._converters[key].call(this, sourceData)
			} else {
				const convertFn = commonConvertMap[key]
				if (convertFn) return convertFn(sourceData)
				return sourceData
			}
		},
		registerEvents() {
			if (!this.$options.propsData) return
			if (this.$options.propsData.events) {
				for (let eventName in this.events) {
					this.$amapComponent.on(eventName, this.events[eventName])
				}
			}
		},
		unregisterEvents() {
			if (this.events) {
				for (let eventName in this.events) {
					this.$amapComponent.off(eventName, this.events[eventName])
				}
			}
		},
		//收集属性，或者事件 监听函数
		setPropWatchers() {
			const {
				$options: { propsData = {} },
			} = this
			console.log(propsData)
			Object.keys(propsData).forEach(prop => {
				let handleProp = prop
				let handleFun = this.getHandlerFun(handleProp)
				//props 为 events 或 prop 对应处理函数
				if (!handleFun && prop !== 'events') return
				const unwatch = this.$watch(prop, nv => {
					if (prop === 'events') {
						this.unregisterEvents()
						this.registerEvents()
						return
					}
					handleFun.call(this.$amapComponent, this.convertSignalProp(prop, nv))
				})
				this.unwatchFns.push(unwatch)
			})
		},
	},
	//清场设置
	destroyed() {
		this.unregisterEvents()
		if (!this.$amapComponent) return
		this.$amapComponent.setMap && this.$amapComponent.setMap(null)
		this.$amapComponent.close && this.$amapComponent.close()
		this.$amapComponent.editor && this.$amapComponent.editor.close()
		this.unwatchFns.forEach(item => item())
		this.unwatchFns = []
	},
}
